package com.wacom.signpro.pdf.api.example;

import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.net.Uri;
import android.os.Build;
import android.os.Environment;
import android.os.ParcelFileDescriptor;
import android.support.v7.app.AlertDialog;
import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;
import android.util.DisplayMetrics;
import android.util.Pair;
import android.util.Size;
import android.util.TypedValue;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.Switch;
import android.widget.TextView;

import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import com.wacom.signpro.pdf.api.example.json.Authentication;
import com.wacom.signpro.pdf.api.example.json.Certificate;
import com.wacom.signpro.pdf.api.example.json.CheckBoxField;
import com.wacom.signpro.pdf.api.example.json.Configuration;
import com.wacom.signpro.pdf.api.example.json.Image;
import com.wacom.signpro.pdf.api.example.json.InitialField;
import com.wacom.signpro.pdf.api.example.json.Input;
import com.wacom.signpro.pdf.api.example.json.JSONParam;
import com.wacom.signpro.pdf.api.example.json.Location;
import com.wacom.signpro.pdf.api.example.json.Output;
import com.wacom.signpro.pdf.api.example.json.SignatureField;
import com.wacom.signpro.pdf.api.example.json.TextField;

import java.io.File;
import java.io.FileDescriptor;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;

public class MainActivity extends AppCompatActivity {

    private static final int PICK_FILE_REQUEST = 1;
    private static final int OPEN_PDF_REQUEST = 2;
    private static final int GENERATE_PNG = 3;
    private static final int SELECT_CERTIFICATE_REQUEST = 4;
    private static final int SELECT_IMAGE_REQUEST = 5;
    private static final int CREATE_DOCUMENT_REQUEST = 6;

    private LinearLayout signatureContainer;
    private LinearLayout initialContainer;
    private LinearLayout textFieldContainer;
    private LinearLayout checkBoxContainer;
    private List<View> signatureList = new LinkedList<>();
    private List<View> initialList = new LinkedList<>();
    private List<View> textFieldList = new LinkedList<>();
    private List<View> checkBoxList = new LinkedList<>();

    private ViewGroup viewGroup;

    private int signatureIndex = 1;
    private int initialIndex = 1;
    private int textFieldIndex = 1;
    private int checkBoxIndex = 1;

    private TextView pdfInputFileText;
    private TextView pdfOutFileText;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);
        signatureContainer = findViewById(R.id.signature_container);
        initialContainer = findViewById(R.id.initial_container);
        textFieldContainer = findViewById(R.id.text_field_container);
        checkBoxContainer = findViewById(R.id.check_box_container);
        pdfInputFileText = findViewById(R.id.file_txt);
        pdfOutFileText = findViewById(R.id.output_name_txt);
    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        if (requestCode == PICK_FILE_REQUEST) {
            if (resultCode == RESULT_OK) {
                getContentResolver().takePersistableUriPermission(data.getData(), Intent.FLAG_GRANT_WRITE_URI_PERMISSION);
                pdfInputFileText.setText(data.getData().toString());
            }
        } else if (requestCode == OPEN_PDF_REQUEST) {
            if ((resultCode == RESULT_OK) && (!pdfOutFileText.getText().toString().isEmpty())) {
                findViewById(R.id.btn_open_pdf).setEnabled(true);
                findViewById(R.id.btn_generate_png).setEnabled(true);
            }
        } else if (requestCode == GENERATE_PNG) {
            if (resultCode == RESULT_OK) {
                byte[] byteArray = data.getByteArrayExtra("png_image");
                Bitmap bmp = BitmapFactory.decodeByteArray(byteArray, 0, byteArray.length);
                openImage(bmp);
            }
        } else if (requestCode == SELECT_CERTIFICATE_REQUEST) {
            if (resultCode == RESULT_OK) {
                if (viewGroup != null) {
                    TextView textView = viewGroup.findViewById(R.id.certificate_path_txt);
                    if (textView != null) {
                        textView.setText(data.getData().toString());
                    }
                }
            }
        } else if (requestCode == SELECT_IMAGE_REQUEST) {
            if (resultCode == RESULT_OK) {
                getContentResolver().takePersistableUriPermission(data.getData(), Intent.FLAG_GRANT_WRITE_URI_PERMISSION);
                if (viewGroup != null) {
                    EditText textView = viewGroup.findViewById(R.id.image_path_txt);
                    Pair<Float, Float> size = getIMGSize(data.getData());

                    if (textView != null) {
                        textView.setText(data.getData().toString());

                        ViewGroup parent = (ViewGroup) viewGroup.getParent();
                        EditText imageWidthTextView = parent.findViewById(R.id.image_width_txt);
                        EditText imageHeightTextView = parent.findViewById(R.id.image_height_txt);

                        if (imageWidthTextView != null) {
                            imageWidthTextView.setText(String.valueOf(size.first));
                        }

                        if (imageHeightTextView != null) {
                            imageHeightTextView.setText(String.valueOf(size.second));
                        }
                    }
                }
            }
        } else if (requestCode == CREATE_DOCUMENT_REQUEST) {
            if (resultCode == RESULT_OK) {
                getContentResolver().takePersistableUriPermission(data.getData(), Intent.FLAG_GRANT_WRITE_URI_PERMISSION);
                pdfOutFileText.setText(data.getData().toString());
            }
        }
    }

    public void selectInputFile(View v) {
        Intent intent = new Intent(Intent.ACTION_OPEN_DOCUMENT);
        intent.setType("application/pdf");
        intent.addCategory(Intent.CATEGORY_OPENABLE);
        startActivityForResult(intent, PICK_FILE_REQUEST);
    }

    public void selectOutputFile(View v) {
        Intent intent = new Intent(Intent.ACTION_CREATE_DOCUMENT);
        intent.setType("application/pdf");
        startActivityForResult(intent, CREATE_DOCUMENT_REQUEST);
    }

    public void addSignature(View v) {
        // Generate dynamic layout
        LayoutInflater layoutInflater = (LayoutInflater) getBaseContext().getSystemService(Context.LAYOUT_INFLATER_SERVICE);
        // Use demo.xml to create layout
        final View addView = layoutInflater.inflate(R.layout.signature_layout, null);
        LinearLayout.LayoutParams layoutParams = new LinearLayout.LayoutParams(LinearLayout.LayoutParams.MATCH_PARENT, LinearLayout.LayoutParams.WRAP_CONTENT);
        layoutParams.setMargins(0, 0, 0, 15);
        addView.setLayoutParams(layoutParams);

        Button deleteBtn = addView.findViewById(R.id.btn_delete_signature);
        deleteBtn.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                signatureList.remove(addView);
                signatureContainer.removeView(addView);
            }
        });

        EditText nameText = addView.findViewById(R.id.name_txt);
        nameText.setText("signature_" + signatureIndex++);

        signatureContainer.addView(addView);
        signatureList.add(addView);
    }

    public void addInitial(View v) {
        // Generate dynamic layout
        LayoutInflater layoutInflater = (LayoutInflater) getBaseContext().getSystemService(Context.LAYOUT_INFLATER_SERVICE);
        // Use demo.xml to create layout
        final View addView = layoutInflater.inflate(R.layout.initial_layout, null);
        LinearLayout.LayoutParams layoutParams = new LinearLayout.LayoutParams(LinearLayout.LayoutParams.MATCH_PARENT, LinearLayout.LayoutParams.WRAP_CONTENT);
        layoutParams.setMargins(0, 0, 0, 15);
        addView.setLayoutParams(layoutParams);

        Button deleteBtn = addView.findViewById(R.id.btn_delete_initial);
        deleteBtn.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                initialList.remove(addView);
                initialContainer.removeView(addView);
            }
        });

        EditText nameText = addView.findViewById(R.id.name_txt);
        nameText.setText("initial_" + initialIndex++);

        initialContainer.addView(addView);
        initialList.add(addView);
    }

    public void addTextField(View v) {
        // Generate dynamic layout
        LayoutInflater layoutInflater = (LayoutInflater) getBaseContext().getSystemService(Context.LAYOUT_INFLATER_SERVICE);
        // Use demo.xml to create layout
        final View addView = layoutInflater.inflate(R.layout.text_field_layout, null);
        LinearLayout.LayoutParams layoutParams = new LinearLayout.LayoutParams(LinearLayout.LayoutParams.MATCH_PARENT, LinearLayout.LayoutParams.WRAP_CONTENT);
        layoutParams.setMargins(0, 0, 0, 15);
        addView.setLayoutParams(layoutParams);

        Button deleteBtn = addView.findViewById(R.id.btn_delete_text_field);
        deleteBtn.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                textFieldList.remove(addView);
                textFieldContainer.removeView(addView);
            }
        });

        EditText nameText = addView.findViewById(R.id.name_txt);
        nameText.setText("textField_" + textFieldIndex++);

        textFieldContainer.addView(addView);
        textFieldList.add(addView);
    }

    public void addCheckBox(View v) {
        // Generate dynamic layout
        LayoutInflater layoutInflater = (LayoutInflater) getBaseContext().getSystemService(Context.LAYOUT_INFLATER_SERVICE);
        // Use demo.xml to create layout
        final View addView = layoutInflater.inflate(R.layout.check_box_layout, null);
        LinearLayout.LayoutParams layoutParams = new LinearLayout.LayoutParams(LinearLayout.LayoutParams.MATCH_PARENT, LinearLayout.LayoutParams.WRAP_CONTENT);
        layoutParams.setMargins(0, 0, 0, 15);
        addView.setLayoutParams(layoutParams);

        Button deleteBtn = addView.findViewById(R.id.btn_delete_check_box);
        deleteBtn.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                checkBoxList.remove(addView);
                checkBoxContainer.removeView(addView);
            }
        });

        EditText nameText = addView.findViewById(R.id.name_txt);
        nameText.setText("checkBox_" + checkBoxIndex++);

        checkBoxContainer.addView(addView);
        checkBoxList.add(addView);
    }

    public void callSignpro(View v) {
        EditText passwordText = findViewById(R.id.password_txt);
        Switch annotationSwitch = findViewById(R.id.annotation_switch);
        Switch manualSwitch = findViewById(R.id.manutal_signatures_switch);
        Switch textTagsSwitch = findViewById(R.id.process_text_tags_switch);

        JSONParam jsonParam = new JSONParam();
        com.wacom.signpro.pdf.api.example.json.File file = new com.wacom.signpro.pdf.api.example.json.File();
        Input input = new Input();
        input.setFilesystem(pdfInputFileText.getText().toString());
        file.setInput(input);
        Authentication authentication = new Authentication();
        authentication.setPdfUserPassword(passwordText.getText().toString());
        file.setAuthentication(authentication);

        String fileName = pdfOutFileText.getText().toString();
        if (!fileName.isEmpty()) {
            Output output = new Output();
            output.setFilesystem(fileName);
            file.setOutput(output);
        }
        jsonParam.setFile(file);

        Configuration configuration = new Configuration();
        configuration.setShowAnnotate(annotationSwitch.isChecked());
        configuration.setShowManualSignature(manualSwitch.isChecked());
        configuration.setProcessTextTags(textTagsSwitch.isChecked());
        jsonParam.setConfiguration(configuration);

        List<SignatureField> signatureFieldList = new LinkedList<>();
        for (View signatureLayout : signatureList) {
            SignatureField signatureField = new SignatureField();

            EditText nameText = signatureLayout.findViewById(R.id.name_txt);
            EditText signerText = signatureLayout.findViewById(R.id.signer_txt);
            EditText reasonText = signatureLayout.findViewById(R.id.reason_txt);
            Switch biometricsSwitch = signatureLayout.findViewById(R.id.bimotrics_switch);
            Switch isRequiredSwitch = signatureLayout.findViewById(R.id.required_switch);
            EditText pageText = signatureLayout.findViewById(R.id.page_txt);
            EditText xText = signatureLayout.findViewById(R.id.x_txt);
            EditText yText = signatureLayout.findViewById(R.id.y_txt);
            EditText wText = signatureLayout.findViewById(R.id.width_txt);
            EditText hText = signatureLayout.findViewById(R.id.height_txt);
            EditText certificatePathText = signatureLayout.findViewById(R.id.certificate_path_txt);
            EditText certificatePasswordText = signatureLayout.findViewById(R.id.certificate_password_txt);
            EditText imagePathText = signatureLayout.findViewById(R.id.image_path_txt);
            EditText imageWidthText = signatureLayout.findViewById(R.id.image_width_txt);
            EditText imageHeightText = signatureLayout.findViewById(R.id.image_height_txt);

            signatureField.setName(nameText.getText().toString());
            signatureField.setSigner(signerText.getText().toString());
            signatureField.setReason(reasonText.getText().toString());
            signatureField.setBiometric(biometricsSwitch.isChecked());
            signatureField.setRequired(isRequiredSwitch.isChecked());

            Location location = new Location();
            location.setPage(pageText.getText().toString());
            location.setX(xText.getText().toString());
            location.setY(yText.getText().toString());
            location.setW(wText.getText().toString());
            location.setH(hText.getText().toString());

            String certificatePath = certificatePathText.getText().toString();
            if (!certificatePath.isEmpty()) {
                Certificate certificate = new Certificate();
                grantUriPermission("com.wacom.signpropdf", Uri.parse(certificatePath), Intent.FLAG_GRANT_READ_URI_PERMISSION);
                certificate.setKeyStorePath(certificatePath);
                certificate.setKeyStorePassword(certificatePasswordText.getText().toString());
                signatureField.setCertificate(certificate);
            }

            String imagePath = imagePathText.getText().toString();
            if (!imagePath.isEmpty()) {
                Image image = new Image();
                image.setFilePath(imagePath);
                grantUriPermission("com.wacom.signpropdf", Uri.parse(imagePath), Intent.FLAG_GRANT_READ_URI_PERMISSION);
                image.setWidth(Integer.parseInt(imageWidthText.getText().toString()));
                image.setHeight(Integer.parseInt(imageHeightText.getText().toString()));
                signatureField.setImage(image);
            }

            signatureField.setLocation(location);
            signatureFieldList.add(signatureField);
        }
        jsonParam.setSignatures(signatureFieldList);

        List<InitialField> initialFieldList = new LinkedList<>();
        for (View initialLayout : initialList) {
            InitialField initialField = new InitialField();

            EditText nameText = initialLayout.findViewById(R.id.name_txt);
            EditText signerText = initialLayout.findViewById(R.id.signer_txt);
            EditText reasonText = initialLayout.findViewById(R.id.reason_txt);
            Switch biometricsSwitch = initialLayout.findViewById(R.id.bimotrics_switch);
            Switch isRequiredSwitch = initialLayout.findViewById(R.id.required_switch);
            EditText pageText = initialLayout.findViewById(R.id.page_txt);
            EditText xText = initialLayout.findViewById(R.id.x_txt);
            EditText yText = initialLayout.findViewById(R.id.y_txt);
            EditText wText = initialLayout.findViewById(R.id.width_txt);
            EditText hText = initialLayout.findViewById(R.id.height_txt);
            EditText certificatePathText = initialLayout.findViewById(R.id.certificate_path_txt);
            EditText certificatePasswordText = initialLayout.findViewById(R.id.certificate_password_txt);
            EditText imagePathText = initialLayout.findViewById(R.id.image_path_txt);
            EditText imageWidthText = initialLayout.findViewById(R.id.image_width_txt);
            EditText imageHeightText = initialLayout.findViewById(R.id.image_height_txt);

            initialField.setName(nameText.getText().toString());
            initialField.setSigner(signerText.getText().toString());
            initialField.setReason(reasonText.getText().toString());
            initialField.setBiometric(biometricsSwitch.isChecked());
            initialField.setRequired(isRequiredSwitch.isChecked());

            Location location = new Location();
            location.setPage(pageText.getText().toString());
            location.setX(xText.getText().toString());
            location.setY(yText.getText().toString());
            location.setW(wText.getText().toString());
            location.setH(hText.getText().toString());

            String certificatePath = certificatePathText.getText().toString();
            if (!certificatePath.isEmpty()) {
                Certificate certificate = new Certificate();
                grantUriPermission("com.wacom.signpropdf", Uri.parse(certificatePath), Intent.FLAG_GRANT_READ_URI_PERMISSION);
                certificate.setKeyStorePath(certificatePath);
                certificate.setKeyStorePassword(certificatePasswordText.getText().toString());
                initialField.setCertificate(certificate);
            }

            String imagePath = imagePathText.getText().toString();
            if (!imagePath.isEmpty()) {
                Image image = new Image();
                image.setFilePath(imagePath);
                grantUriPermission("com.wacom.signpropdf", Uri.parse(imagePath), Intent.FLAG_GRANT_READ_URI_PERMISSION);
                image.setWidth(Integer.parseInt(imageWidthText.getText().toString()));
                image.setHeight(Integer.parseInt(imageHeightText.getText().toString()));
                initialField.setImage(image);
            }

            initialField.setLocation(location);
            initialFieldList.add(initialField);
        }
        jsonParam.setInitials(initialFieldList);

        List<TextField> textList = new LinkedList<>();
        for (View textFieldLayout : textFieldList) {
            TextField textField = new TextField();

            EditText nameText = textFieldLayout.findViewById(R.id.name_txt);
            EditText valueText = textFieldLayout.findViewById(R.id.value_txt);
            EditText pageText = textFieldLayout.findViewById(R.id.page_txt);
            EditText xText = textFieldLayout.findViewById(R.id.x_txt);
            EditText yText = textFieldLayout.findViewById(R.id.y_txt);
            EditText wText = textFieldLayout.findViewById(R.id.width_txt);
            EditText hText = textFieldLayout.findViewById(R.id.height_txt);

            textField.setName(nameText.getText().toString());
            textField.setValue(valueText.getText().toString());

            Location location = new Location();
            location.setPage(pageText.getText().toString());
            location.setX(xText.getText().toString());
            location.setY(yText.getText().toString());
            location.setW(wText.getText().toString());
            location.setH(hText.getText().toString());

            textField.setLocation(location);
            textList.add(textField);
        }
        jsonParam.setTextFields(textList);

        List<CheckBoxField> checkBoxFieldList = new LinkedList<>();
        for (View checkBoxLayout : checkBoxList) {
            CheckBoxField checkBoxField = new CheckBoxField();

            EditText nameText = checkBoxLayout.findViewById(R.id.name_txt);
            Switch valueSwitch = checkBoxLayout.findViewById(R.id.check_box_switch);
            EditText pageText = checkBoxLayout.findViewById(R.id.page_txt);
            EditText xText = checkBoxLayout.findViewById(R.id.x_txt);
            EditText yText = checkBoxLayout.findViewById(R.id.y_txt);
            EditText wText = checkBoxLayout.findViewById(R.id.width_txt);
            EditText hText = checkBoxLayout.findViewById(R.id.height_txt);

            checkBoxField.setName(nameText.getText().toString());
            checkBoxField.setValue(Boolean.toString(valueSwitch.isChecked()));

            Location location = new Location();
            location.setPage(pageText.getText().toString());
            location.setX(xText.getText().toString());
            location.setY(yText.getText().toString());
            location.setW(wText.getText().toString());
            location.setH(hText.getText().toString());

            checkBoxField.setLocation(location);
            checkBoxFieldList.add(checkBoxField);
        }
        jsonParam.setCheckBoxes(checkBoxFieldList);

        Gson gson = new GsonBuilder().create();
        String jsonString = gson.toJson(jsonParam, JSONParam.class);
        //Intent intent = new Intent(Intent.ACTION_SEND_MULTIPLE);
        //intent.addFlags(Intent.FLAG_GRANT_WRITE_URI_PERMISSION);
        Intent intent = new Intent("com.wacom.signpro.pdf.SignProPDF");
        intent.setData(Uri.fromParts("signpro", jsonString, null));
        grantUriPermission("com.wacom.signpropdf", Uri.parse(pdfInputFileText.getText().toString()), Intent.FLAG_GRANT_READ_URI_PERMISSION);

        if (!pdfOutFileText.getText().toString().isEmpty()) {
            grantUriPermission("com.wacom.signpropdf", Uri.parse(pdfOutFileText.getText().toString()), Intent.FLAG_GRANT_WRITE_URI_PERMISSION);
        }

        //uris.add(Uri.fromParts("signpro", jsonString, null));
        //uris.add(Uri.parse(pdfInputFileText.getText().toString()));
        //uris.add(Uri.parse(pdfOutFileText.getText().toString()));


        //intent.putParcelableArrayListExtra(Intent.EXTRA_STREAM, uris);
        //intent.setComponent(new ComponentName("com.wacom.signpropdf", "com.wacom.signpropdf.ViewerActivity"));
        //intent.setType("application/json");
        //List<ResolveInfo> infoList = getPackageManager().queryIntentActivities(intent, 0);
        //ResolveInfo resolveInfo = infoList.get(0);
        //intent.setComponent(new ComponentName(resolveInfo.activityInfo.packageName, resolveInfo.activityInfo.name));
        startActivityForResult(intent, OPEN_PDF_REQUEST);


    }

    public void openPDF(View v) {
        Intent intent = new Intent("com.wacom.signpro.pdf.SignProPDF");
        Uri uri = Uri.parse(pdfOutFileText.getText().toString());
        intent.setData(uri);
        grantUriPermission("com.wacom.signpropdf", uri, Intent.FLAG_GRANT_READ_URI_PERMISSION);
        //intent.setType("application/pdf");
        startActivity(intent);
    }

    public void generatePNG(View v) {
        Intent intent = new Intent("com.wacom.signpro.pdf.api");
        intent.putExtra("action", "pdf_to_bmp");
        Uri uri = Uri.parse(pdfOutFileText.getText().toString());
        grantUriPermission("com.wacom.signpropdf", uri, Intent.FLAG_GRANT_READ_URI_PERMISSION);
        intent.putExtra("src_pdf", uri);
        startActivityForResult(intent, GENERATE_PNG);
    }

    public void selectCertificate(View v) {
        viewGroup = (ViewGroup) v.getParent();

        Intent intent = new Intent(Intent.ACTION_OPEN_DOCUMENT);
        intent.setType("application/*");
        intent.addCategory(Intent.CATEGORY_OPENABLE);
        startActivityForResult(intent, SELECT_CERTIFICATE_REQUEST);
    }

    public void selectImage(View v) {
        viewGroup = (ViewGroup) v.getParent();

        Intent intent = new Intent(Intent.ACTION_OPEN_DOCUMENT);
        intent.setType("image/*");
        intent.addCategory(Intent.CATEGORY_OPENABLE);
        startActivityForResult(intent, SELECT_IMAGE_REQUEST);
    }

    private Pair getIMGSize(Uri uri) {
        BitmapFactory.Options options = new BitmapFactory.Options();
        options.inJustDecodeBounds = true;
        try (ParcelFileDescriptor parcelFileDescriptor = getContentResolver().openFileDescriptor(uri, "r")) {
            FileDescriptor fileDescriptor = parcelFileDescriptor.getFileDescriptor();
            BitmapFactory.decodeFileDescriptor(fileDescriptor, null, options);
        } catch (IOException e) {
            e.printStackTrace();
        }

        return new Pair(options.outWidth, options.outHeight);
    }

    private void openImage(Bitmap bmp) {
        AlertDialog.Builder alertadd = new AlertDialog.Builder(this);
        LayoutInflater factory = LayoutInflater.from(this);
        final View view = factory.inflate(R.layout.preview_png, null);
        alertadd.setView(view);

        ImageView preview = view.findViewById(R.id.dialog_imageview);
        preview.setImageBitmap(bmp);
        alertadd.show();
    }
}
